// USBHIDDlg.cpp : implementation file
//

#include "stdafx.h"
#include "USBHID.h"
#include "USBHIDDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUSBHIDDlg dialog

CUSBHIDDlg::CUSBHIDDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CUSBHIDDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CUSBHIDDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CUSBHIDDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CUSBHIDDlg)
	DDX_Control(pDX, IDC_CLEAR, m_Clear);
	DDX_Control(pDX, IDC_RATE, m_Rate);
	DDX_Control(pDX, IDC_EDIT1, m_InputsPage);
	DDX_Control(pDX, IDC_DEVICE_LIST, m_DeviceList);
	DDX_Control(pDX, IDC_REFRESH, m_Refresh);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CUSBHIDDlg, CDialog)
	//{{AFX_MSG_MAP(CUSBHIDDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_REFRESH, OnRefresh)
	ON_CBN_SELCHANGE(IDC_DEVICE_LIST, OnSelchangeDeviceList)
	ON_WM_CLOSE()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUSBHIDDlg message handlers

BOOL CUSBHIDDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here

	m_Refresh.SetIcon(theApp.LoadIcon(IDI_JOYSTICK));
	
	m_DeviceList.AddString("No Devices Detected");
	m_DeviceList.SetCurSel(0);
	m_DeviceList.SetItemData(0, 0);

	m_pDeviceInfo = (CEdit *)GetDlgItem(IDC_DEVINFO);

	m_DevCnt = 0;

	m_InputsPage.SetWindowText("");

	memset (DataBuffer, 0, MAX_INPUTS);
	memset (DataBufferOld, 0, MAX_INPUTS);

	SetTimer(1, 10, 0);		// timer event 1. Read inputs every 10 ms
	SetTimer(2, 1000, 0);	// timer event 2. Update rate monitor

	OnRefresh();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CUSBHIDDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CUSBHIDDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CUSBHIDDlg::OnRefresh() 
{
	CString str;
	int nIndex;

	m_DeviceList.ResetContent();
	if (!DetectHID(&m_DevCnt, DeviceList, DT_ALL)) {
		m_DevCnt = 0;
		m_pDeviceInfo->SetWindowText(_T("No compatible devices detected!"));
		m_DeviceList.AddString("No Devices Detected");
		m_DeviceList.SetCurSel(0);
		m_DeviceList.SetItemData(0, 0);
	}

	// get all port status data if required
//	for (i = 0; i < m_DevCnt; i++) {
//		RetrieveStatus(&DeviceList[i], &DeviceStatus[i]);
//	}

	// Add device to combo list
	for (UINT i = 0; i < m_DevCnt; i++) {
		if (DeviceList[i].NumberInputIndices > 0) { // Add device only in it has Inputs
			str.Format("%s - %s", DeviceList[i].DeviceName, DeviceList[i].SerialNum);
			nIndex = m_DeviceList.AddString(str);
			m_DeviceList.SetItemData(nIndex, i);

			// Display Device info
			DeviceInfo(nIndex);
		}
	}

	m_DeviceList.SetCurSel(0);
	m_DeviceIndex = m_DeviceList.GetItemData(0);
}

void CUSBHIDDlg::OnSelchangeDeviceList() 
{
	m_DeviceIndex = m_DeviceList.GetItemData(m_DeviceList.GetCurSel());
	if (m_DevCnt > 0)
		DeviceInfo(m_DeviceIndex);
}

void CUSBHIDDlg::DeviceInfo(int nIndex)
{
	CString str1, str2, str3, str4, str5, str6, str7;

	// Display Device info
	str1.Format("Device: %d\r\n", nIndex+1);
	str2.Format("  Device Type: %s\r\n", DeviceList[nIndex].DeviceName);
	str3.Format("  Vendor ID: 0x%x\r\n", DeviceList[nIndex].VendorID);
	str4.Format("  Product ID: 0x%x\r\n", DeviceList[nIndex].ProductID);
	str5.Format("  Firmware Version: %d.%d%d\r\n", 
		(((int)(DeviceList[nIndex].VersionNumber & 0xF000)>>12) * 10) + 
		((DeviceList[nIndex].VersionNumber & 0x0F00)>>8),
		(DeviceList[nIndex].VersionNumber & 0x00F0)>>4,
		DeviceList[nIndex].VersionNumber & 0x000F);
	str6.Format("  Serial Num: %s\r\n", DeviceList[nIndex].SerialNum);
	m_pDeviceInfo->SetWindowText(str1 + str2 + str3 + str4 + str5 + str6);
}

void CUSBHIDDlg::OnClose() 
{
	if (m_DevCnt > 0)
		CloseDevices(m_DevCnt, DeviceList);
	
	CDialog::OnClose();
}

void CUSBHIDDlg::OnTimer(UINT nIDEvent) 
{
	CString str, str2;
	long retval;
	int i;
	static long DeltaT, DeltaTOld;
	SYSTEMTIME		mySYSTEMTIME;

	if (nIDEvent == 1) {
		// This is our timer. 10 ms Elapsed.
		// Read inputs into buffer.
		retval = ReadInputData(&DeviceList[m_DeviceIndex], DataBuffer, FALSE);

		switch(retval) {
			case DEV_WAIT:
				// nothing to do but wait...
				break;

			case DEV_INPUT:
				// new data returned.

				GetSystemTime(&mySYSTEMTIME);
				DeltaT = (USHORT)(mySYSTEMTIME.wSecond*1000 + mySYSTEMTIME.wMilliseconds);
				DeltaT = DeltaT - DeltaTOld;
				DeltaTOld = (USHORT)(mySYSTEMTIME.wSecond*1000 + mySYSTEMTIME.wMilliseconds);;

				// loop through inputs and look for any toggled inputs.
				for (i = 0; i < DeviceList[m_DeviceIndex].NumberInputIndices; i++) {
					if (DataBuffer[i] != DataBufferOld[i]) {
						DataBufferOld[i] = DataBuffer[i];

						// display change
						m_InputsPage.GetWindowText(str2);
						str.Format ("%s - Input %d : %s\r\n%s", DeviceList[m_DeviceIndex].DeviceName, i+1, (DataBuffer[i] ? "ON":"OFF"), str2);
						m_InputsPage.SetWindowText(str);
					}
				}
				break;

			case DEV_TIMEOUT:
				// something went wrong. The device may be inoperable or disconnected
				//timeval = 0;
				break;
		}
	} else if (nIDEvent == 2) {
		// update rate monitor
		str.Format("Input Rate : %dms", DeltaT);
		m_Rate.SetWindowText(str);
	}
	
	CDialog::OnTimer(nIDEvent);
}

void CUSBHIDDlg::OnClear() 
{
	m_InputsPage.SetWindowText("");
}
