Attribute VB_Name = "ApiDeclarations"
'******************************************************************************
' API constants
'******************************************************************************

Public Const MAX_DEVICE_COUNT = 128
Public Const MAX_STRING_CHARS = 255
Public Const MAX_DEVICES = 128
Public Const MAX_PORTS = 24
Public Const MAX_INPUTS = 512
Public Const MAX_OUTPUTS = MAX_PORTS * 15
Public Const MAX_ANALOG = 8
Public Const MAX_HATS = 4
Public Const MAX_ROTARY = 200
Public Const MAX_PWM = 512
Public Const MAX_LCDS = 200
Public Const MAX_MUXDISPLAYS = 2000
Public Const MAX_ALPHANUMERICS = 100
Public Const MAX_DACS = MAX_PORTS * 10
Public Const MAX_SPI = MAX_PORTS * 10
Public Const MAX_LATCHED = MAX_PORTS * 64
Public Const MAX_RWR = MAX_PORTS
Public Const MAX_RWR_CONTACTS = 40
Public Const MAX_GLCDS = MAX_PORTS
Public Const MAX_DOTMATRIX = MAX_PORTS * 4

' USB PRODUCT IDs : HID DEVICES (DT_HID)
Public Const ID_0X64B = &H300           ' 0X64B
Public Const ID_GAMMARAY256 = &H500     ' GammaRay-256
Public Const ID_GAMMARAY_V2 = &H510     ' GammaRay V2
Public Const ID_GAMMARAY_V3 = &H520     ' GammaRay V3
Public Const ID_GAMMARAY64 = &H700      ' GammaRay-64
Public Const ID_GAMMATRON = &H710       ' GammaTron
Public Const ID_ELECTRONFLUX = &H800    ' ElectronFlux
Public Const ID_NITRO_SLG = &H1500      ' NITRO-SLG

' USB PRODUCT IDs : STANDARD JOYSTICK DEVICES (DT_DEVICES)
Public Const ID_4X24BH = &H100          ' 4X24BH
Public Const ID_6X16B = &H200           ' 6X16B
Public Const ID_6X13B = &H250           ' 6X13B
Public Const ID_5X18BH = &H400          ' 5X18BH
Public Const ID_PLASMA = &H600          ' PLASMA SINGLE MODE
Public Const ID_PLASMA_HOTAS = &H601    ' PLASMA HOTAS SINGLE MODE
Public Const ID_PLASMA_DUAL = &H602     ' PLASMA DUAL MODE
Public Const ID_PLASMA_HOTAS_DUAL = &H603   ' PLASMA HOTAS DUAL MODE
Public Const ID_PLASMA_LITE = &H900     ' PLASMA-LITE
Public Const ID_PLASMA_LITE_V2 = &H910  ' PLASMA-LITE V2
Public Const ID_PLASMA_MM2 = &H920      ' PLASMA-MM2

' RETURN CONSTANTS
Public Const DEV_FAILED = &H1
Public Const DEV_INPUT = &H2
Public Const DEV_WAIT = &H3
Public Const DEV_TIMEOUT = &H4

' API CONSTANTS
Public Const DT_DEVICES = &H2
Public Const DT_HID = &H4
Public Const DT_ALL = &HFF

' LCD Commands
Public Const LCD_CLEAR = &H0
Public Const LCD_PAGE1 = &H1
Public Const LCD_PAGE2 = &H2
Public Const LCD_TEST = &H3

' GRAPHICAL LCD Commands
Public Const GLCD_CLEAR = 0
Public Const GLCD_TEST = 1
Public Const GLCD_DISPLAY = 2
Public Const GLCD_BGCOLOR = 3

' PortMode CONSTANTS
Public Const MODE_DIRECT = 0
Public Const MODE_MUX = 1
Public Const MODE_RWR = 2
Public Const MODE_GLCD = 3
Public Const MODE_DAC = 4
Public Const MODE_LATCHED = 5
Public Const MODE_SPI = 6
Public Const MODE_DOTMATRIX = 7

'******************************************************************************
' User-defined types for API calls
'******************************************************************************

Public Type DeviceStatus
    InputActive(MAX_INPUTS) As Byte
    OutputActive(MAX_OUTPUTS) As Byte
    PortActive(MAX_PORTS) As Byte
    PortIOMode(MAX_PORTS) As Byte
    PortMode(MAX_PORTS) As Byte
    OutputMode(MAX_PORTS) As Byte
    AnalogActive(MAX_ANALOG) As Byte
    HATActive(MAX_HATS) As Byte
    RotaryActive(MAX_ROTARY) As Byte
    CRTRWRActive(MAX_RWR) As Byte
    PWMActive(MAX_PWM) As Byte
    LCDActive(MAX_LCDS) As Byte
    GLCDActive(MAX_GLCDS) As Byte
    MuxDisplayActive(MAX_MUXDISPLAYS) As Byte
    AlphanumericActive(MAX_ALPHANUMERICS) As Byte
    DACActive(MAX_DACS) As Byte
    SPIActive(MAX_SPI) As Byte
    LatchedActive(MAX_LATCHED) As Byte
    DotMatrixActive(MAX_DOTMATRIX) As Byte
End Type


Public Type DeviceParam
    DeviceHandle As Long
    NumberInputIndices As Integer
    NumberOutputIndices As Integer
    NumberPortIndices As Integer
    NumberAnalogIndices As Integer
    NumberHATIndices As Integer
    NumberRotaryIndices As Integer
    NumberCRTRWRIndices As Integer
    NumberPWMIndices As Integer
    NumberLCDIndices As Integer
    NumberGLCDIndices As Integer
    NumberMuxDisplayIndices As Integer
    NumberAlphanumericIndices As Integer
    NumberDACIndices As Integer
    NumberSPIIndices As Integer
    NumberLatchedIndices As Integer
    NumberDotMatrixIndices As Integer
    VendorID As Integer
    ProductID As Integer
    VersionNumber As Integer
    FlashVersion As Integer
    ProgramFlag As Byte
    DevicePath(MAX_STRING_CHARS) As Byte
    DevicePathLength As Long
    DeviceName(MAX_STRING_CHARS) As Byte
    DeviceNameLength As Long
    ManufName(MAX_STRING_CHARS) As Byte
    ManufNameLength As Long
    SerialNum(0 To 3) As Byte
    SerialNumLength As Long
    ConfigFlag As Byte
    DevIndex As Integer
    InputReportByteLength As Integer
    OutputReportByteLength As Integer
    Usage As Integer
    UsagePage As Integer
End Type

Public Type DIRECT_OUTPUT
    state As Byte
End Type

Public Type DAC_OUTPUTS
    DAC_1 As Integer
    DAC_2 As Integer
    DAC_3 As Integer
    DAC_4 As Integer
    DAC_5 As Integer
    DAC_6 As Integer
    DAC_7 As Integer
    DAC_8 As Integer
    DAC_9 As Integer
    DAC_10 As Integer
End Type

Public Type CRTRWR_OUTPUT
    symbol As Byte
    xPos As Byte
    yPos As Byte
    options As Byte
    ' missileActivity bit 0 - draw flashing circle
    ' missileLaunch bit 1 - draw solid circle
    ' newguy bit 2
    ' selected bit 3 - draw diamond if 1
End Type

'******************************************************************************
' User-defined contants and variables
'******************************************************************************

Public CurrentIndex As Integer
Public DeviceList(MAX_DEVICES) As DeviceParam
Public DeviceStatus(MAX_DEVICES) As DeviceStatus
Public Dev_Cnt As Integer

'******************************************************************************
' API functions
'******************************************************************************

Public Declare Function ReadInputData Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByRef oDataBuffer As Byte, ByVal iFlag As Long) As Long
Public Declare Function CmdGLCD Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal inPort As Byte, ByVal iBGColor As Byte, ByVal iCmd As Byte) As Long
Public Declare Function WriteGLCD Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal nPort As Byte, ByVal nLine As Byte, ByVal nNumLines As Byte, ByVal nBGColor As Byte, ByRef oDataBuffer As Byte) As Long
Public Declare Function ClearLCD Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal inLCD As Byte) As Long
Public Declare Function WriteLCD Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal inLCD As Byte, ByVal inLine As Byte, ByRef iDataBuffer As Byte, ByVal inLevel As Byte) As Long
Public Declare Function WriteDirectOutput Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal inPort As Byte, ByRef iDataBuffer As DIRECT_OUTPUT) As Long
Public Declare Function WriteMuxOutput Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal nPort As Byte, ByRef iDataBuffer As DIRECT_OUTPUT) As Long
Public Declare Function WriteDisplayOutput Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal inPort As Byte, ByRef iDataBuffer As Byte, ByVal inLevel As Byte) As Long
Public Declare Function Write7Segment Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByRef iData As Byte) As Long
Public Declare Function WriteSPIDAC Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal iPort As Byte, ByRef iDataBuffer As DAC_OUTPUTS, ByVal inLevel As Byte) As Long
Public Declare Function WriteLatchedOutput Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal iPort As Byte, ByRef iDataBuffer As Byte, ByVal inLevel As Byte) As Long
Public Declare Function WriteCRTRWR Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal iPort As Byte, ByVal inContacts As Byte, ByVal inSymbolCnt As Byte, ByVal inContactIndex As Byte, ByRef iDataBuffer As CRTRWR_OUTPUT) As Long
Public Declare Function WriteDotMatrix Lib "biusb.dll" (ByRef iDeviceList As DeviceParam, ByVal iPort As Byte, ByRef iDataBuffer As Byte, ByVal inLevel As Byte) As Long

'******************************************************************************
' DLL functions
'******************************************************************************

Public Declare Function DetectHID Lib "biusb.dll" (ByRef oDev_Cnt As Integer, ByRef opDeviceList As DeviceParam, ByVal iFlag As Long) As Boolean
Public Declare Function DetectDevice Lib "biusb.dll" (ByRef oDev_Cnt As Integer, ByRef opDeviceList As DeviceParam, ByVal iProductID As Long, ByVal iFlag As Long) As Boolean
Public Declare Sub RetrieveStatus Lib "biusb.dll" (ByRef oDeviceList As DeviceParam, ByRef oDeviceStatus As DeviceStatus)
Public Declare Function CloseDevices Lib "biusb.dll" (ByVal iDev_Cnt As Long, ByRef iDeviceList As DeviceParam) As Boolean
